//
//  WDInstallProcessController.h
//  Installer
//
//  Created by Clark Jackson on 12/9/10.
//  Copyright 2013 Western Digital Technologies, Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "WDAuthorizedTask.h"


APPKIT_EXTERN NSString* const WDSyncServiceAppName;
APPKIT_EXTERN NSString* const WDResourcesTmpInstallDirPathKey;
APPKIT_EXTERN NSString* const CHMODsuArrayKey;
APPKIT_EXTERN NSString* const CHMODallArrayKey;
APPKIT_EXTERN NSString* const CHOWNrootArrayKey;
APPKIT_EXTERN NSString* const UnloadArrayKey;
APPKIT_EXTERN NSString* const LoadArrayKey;
APPKIT_EXTERN NSString* const LoadArrayUnprivilegedKey;
APPKIT_EXTERN NSString* const UnloadArrayUnprivilegedKey;
APPKIT_EXTERN NSString* const CopyFromArrayKey;
APPKIT_EXTERN NSString* const CopyToArrayKey;
APPKIT_EXTERN NSString* const DeleteArraysKey;
APPKIT_EXTERN NSString* const KillProcessArrayKey;
APPKIT_EXTERN NSString* const AutoStartArrayKey;
APPKIT_EXTERN NSString* const LaunchListArrayKey;
APPKIT_EXTERN NSString* const CleanupArrayKey;
APPKIT_EXTERN NSString* const ReducePrivilegeArrayKey;
APPKIT_EXTERN NSString* const CreateDirsUnprivilegedArrayKey;
APPKIT_EXTERN NSString* const CreateDirsPrivilegedArrayKey;
APPKIT_EXTERN NSString* const CopyArrayKey;
APPKIT_EXTERN NSString* const UninstallerPathsArrayKey;
APPKIT_EXTERN NSString* const AliasPrefixArrayKey;
APPKIT_EXTERN NSString* const RemovePluginArrayKey;


typedef enum isAliasStates
{
    kUnknownIfAlias,
    kIsAnAlias,
    kIsNotAnAlias
} AliasStates;

typedef enum cancelTypes
{
    kCancelledOSAuthorization,
    kCancelledInProcess
} CancelTypes;


@protocol ApolloProtocol<NSObject>

- (void) doStateChange:(int)state;
- (void) presentThreadedInstallProgress;
- (void) setInstallProgress:(double) progress;
- (BOOL) bypassAuthorizationDialogFlag;
- (void) finishInstallByFramework;
- (void) finishUninstallByFramework;
- (void) cancelInstallByFramework:(int)type;
- (void) cancelUninstallByFramework;

@end



@interface WDInstallProcessController : WDAuthorizedTask<ApolloProtocol>
{
    NSDictionary* processControlDict;
    BOOL shutdownServices;
    BOOL installCancelled;
    id callerController;
    BOOL forceFailure;    
}

@property (readwrite, copy) NSDictionary* processControlDict;
@property (readwrite, retain) id callerController;
@property (readwrite, assign) BOOL installCancelled;
@property (readwrite, assign) BOOL forceFailure;

- (id) initWithPlanDict:(NSDictionary*)installDict;
- (void) setupInstallDict:(NSDictionary*)installDict;
- (void) processControlDictionary;
+ (BOOL) compareInstalledVersion:(NSString*) installedVersionString withUpdateVersion:(NSString*)updateVersionString;
- (void) killUnprivilegedApp:(NSString*)appName;
+ (int) getPIDForProcess:(NSString*) appName;
+ (NSInteger) isAlias:(NSString*)path;



#pragma mark autolaunching
- (void) removeAutoLaunchOfAppPath:(NSString*)appPath locally:(BOOL) doItForSession; // appPath should look like @"/Applications/WD Sync.app"
- (void) removeAutoLaunchOfAppName:(NSString *)appName locally:(BOOL) doItForSession; // appPath should look like @"WD Sync.app"
- (void) setupAutoLaunchOfAppPath:(NSString*)appPath locally:(BOOL) doItForSession; // appPath should look like @"/Applications/WD Sync.app"
- (void) launchApp:(NSString*)appPath launchFlags:(LSLaunchFlags)launchFlags;
- (void) enableLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs ForPath:(NSString *)appPath; // deprecated
- (void) enableLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs ForPath:(NSString *)appPath locally:(BOOL) doItForSession;
- (void) disableLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs ForPath:(NSString *)appPath; // deprecated
- (void) disableLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs ForPath:(NSString *)appPath locally:(BOOL) doItForSession;
- (BOOL) loginItemExistsWithLoginItemReference:(LSSharedFileListRef)theLoginItemsRefs ForPath:(NSString *)appPath;
- (BOOL) loginItemExistsWithLoginItemReference:(LSSharedFileListRef)theLoginItemsRefs WithAppName:(NSString *)appName;
- (void) disableAnyLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs WithAppName:(NSString *)appName; // deprecated
- (void) disableAnyLoginItemWithLoginItemsReference:(LSSharedFileListRef )theLoginItemsRefs WithAppName:(NSString *)appName locally:(BOOL) doItForSession;

#pragma mark favorites list in sidebar
- (void) setupSidebarFavoriteOfPath:(NSString*)favoritePath;

@end



